package com.sun.jmx.mbeanserver;

import java.util.Collections;
import java.security.AccessController;
import java.util.Iterator;
import java.util.Arrays;
import javax.management.InvalidAttributeValueException;
import javax.management.ReflectionException;
import javax.management.MBeanException;
import javax.management.AttributeNotFoundException;
import java.util.List;
import java.util.Map;
import javax.management.MBeanInfo;

final class PerInterface<M>
{
    private final Class<?> mbeanInterface;
    private final MBeanIntrospector<M> introspector;
    private final MBeanInfo mbeanInfo;
    private final Map<String, M> getters;
    private final Map<String, M> setters;
    private final Map<String, List<PerInterface.MethodAndSig>> ops;
    
    PerInterface(Class<?> mbeanInterface, MBeanIntrospector<M> introspector, MBeanAnalyzer<M> mBeanAnalyzer, MBeanInfo mbeanInfo) {
        this.getters = Util.newMap();
        this.setters = Util.newMap();
        this.ops = Util.newMap();
        this.mbeanInterface = mbeanInterface;
        this.introspector = introspector;
        this.mbeanInfo = mbeanInfo;
        mBeanAnalyzer.visit(new PerInterface.InitMaps());
    }
    
    Class<?> getMBeanInterface() {
        return this.mbeanInterface;
    }
    
    MBeanInfo getMBeanInfo() {
        return this.mbeanInfo;
    }
    
    boolean isMXBean() {
        return this.introspector.isMXBean();
    }
    
    Object getAttribute(Object o, String s, Object o2) throws AttributeNotFoundException, MBeanException, ReflectionException {
        Object value = this.getters.get(s);
        if (value == null) {
            String message;
            if (this.setters.containsKey(s)) {
                message = "Write-only attribute: " + s;
            }
            else {
                message = "No such attribute: " + s;
            }
            throw new AttributeNotFoundException(message);
        }
        else {
            return this.introspector.invokeM(value, o, (Object[])null, o2);
        }
    }
    
    void setAttribute(Object o, String s, Object o2, Object o3) throws AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException {
        Object value = this.setters.get(s);
        if (value == null) {
            String message;
            if (this.getters.containsKey(s)) {
                message = "Read-only attribute: " + s;
            }
            else {
                message = "No such attribute: " + s;
            }
            throw new AttributeNotFoundException(message);
        }
        else {
            this.introspector.invokeSetter(s, value, o, o2, o3);
            return;
        }
    }
    
    Object invoke(Object o, String str, Object[] array, String[] a2, Object o2) throws MBeanException, ReflectionException {
        List<Object> list = (List)this.ops.get(str);
        if (list == null) {
            return this.noSuchMethod("No such operation: " + str, o, str, array, a2, o2);
        }
        if (a2 == null) {
            a2 = new String[0];
        }
        PerInterface.MethodAndSig methodAndSig = null;
        for (PerInterface.MethodAndSig methodAndSig2 : list) {
            if (Arrays.equals(methodAndSig2.signature, a2)) {
                methodAndSig = methodAndSig2;
                break;
            }
            continue;
        }
        if (methodAndSig == null) {
            String sigString = this.sigString(a2);
            String s;
            if (list.size() == 1) {
                s = "Signature mismatch for operation " + str + ": " + sigString + " should be " + this.sigString(((PerInterface.MethodAndSig)list.get(0)).signature);
            }
            else {
                s = "Operation " + str + " exists but not with this signature: " + sigString;
            }
            return this.noSuchMethod(s, o, str, array, a2, o2);
        }
        else {
            return this.introspector.invokeM(methodAndSig.method, o, array, o2);
        }
    }
    
    private Object noSuchMethod(String message, Object o, String s, Object[] array, String[] a, Object o2) throws MBeanException, ReflectionException {
        ReflectionException ex = new ReflectionException(new NoSuchMethodException(s + this.sigString(a)), message);
        if (this.introspector.isMXBean()) {
            throw ex;
        }
        GetPropertyAction action = new GetPropertyAction("jmx.invoke.getters");
        String s2;
        try {
            s2 = (String)AccessController.doPrivileged(action);
        }
        catch (Exception ex2) {
            s2 = null;
        }
        if (s2 == null) {
            throw ex;
        }
        int beginIndex = 0;
        Map<Object, Object> map = null;
        if (a == null || a.length == 0) {
            if (s.startsWith("get")) {
                beginIndex = 3;
            }
            else if (s.startsWith("is")) {
                beginIndex = 2;
            }
            if (beginIndex != 0) {
                map = this.getters;
            }
        }
        else if (a.length == 1 && s.startsWith("set")) {
            beginIndex = 3;
            map = this.setters;
        }
        if (beginIndex != 0) {
            Object value = map.get(s.substring(beginIndex));
            if (value != null && this.introspector.getName(value).equals(s)) {
                String[] signature = this.introspector.getSignature(value);
                if ((a == null && signature.length == 0) || Arrays.equals(a, signature)) {
                    return this.introspector.invokeM(value, o, array, o2);
                }
            }
        }
        throw ex;
    }
    
    private String sigString(String[] array) {
        StringBuilder sb = new StringBuilder("(");
        if (array != null) {
            for (String str : array) {
                if (sb.length() > 1) {
                    sb.append(", ");
                }
                sb.append(str);
            }
        }
        return sb.append(")").toString();
    }
    
    private class InitMaps implements MBeanAnalyzer.MBeanVisitor<M>
    {
        @Override
        public void visitAttribute(String s, M m, M i) {
            if (m != null) {
                PerInterface.this.introspector.checkMethod(m);
                Object put = PerInterface.this.getters.put(s, m);
                assert put == null;
            }
            if (i != null) {
                PerInterface.this.introspector.checkMethod(i);
                Object put2 = PerInterface.this.setters.put(s, i);
                assert put2 == null;
            }
        }
        
        @Override
        public void visitOperation(String s, M method) {
            PerInterface.this.introspector.checkMethod(method);
            String[] signature = PerInterface.this.introspector.getSignature(method);
            PerInterface.MethodAndSig o = new PerInterface.MethodAndSig();
            o.method = method;
            o.signature = signature;
            List<PerInterface.MethodAndSig> list = (List)PerInterface.this.ops.get(s);
            if (list == null) {
                list = Collections.singletonList(o);
            }
            else {
                if (list.size() == 1) {
                    list = Util.newList(list);
                }
                list.add(o);
            }
            PerInterface.this.ops.put(s, list);
        }
    }
    
    private class MethodAndSig
    {
        M method;
        String[] signature;
    }
}
